import datetime
from mimetypes import guess_type
from os import fstat

from .constants import status_not_found, status_ok, status_partial_content


def handle_download_requests(request, response, file_path):
    range_from, range_to = request.get_range_header()

    ctype = guess_type(file_path)
    mime_type = ctype[0]

    # send headers
    f = None

    try:

        # Always read in binary mode. Opening files in text mode may cause

        # newline translations, making the actual size of the content

        # transmitted *less* than the content-length!

        f = open(file_path, 'rb')

    except IOError:
        response.status = str(status_not_found)
        response.content = 'File not found'
        return response
    else:
        response.content_type = mime_type

        fs = fstat(f.fileno())

        file_size = fs.st_size

        if range_from is not None:

            response.status = str(status_partial_content)

            if range_to is None or range_to >= file_size:
                range_to = file_size - 1

            # Add 1 because ranges are inclusive
            response.headers = [
                ("Content-Range", "bytes %d-%d/%d" % (range_from, range_to, file_size)),
                ("Content-Length", str((1 + range_to - range_from)))
            ]

        else:
            # 'Sun, 06 Nov 1994 08:49:37 GMT'
            # year = datetime.datetime.fromtimestamp(timestamp).strftime('%Y')
            # return datetime.datetime.strptime(s, '%Y%m%d%H%M').strftime('%m/%d/%y %H:%M')

            response.status = str(status_ok)
            range_from = 0
            range_to = file_size

            response.headers = [
                ("Content-Length", str(file_size)),
                ("Last-Modified", datetime.datetime.fromtimestamp(fs.st_mtime).strftime('%m/%d/%y %H:%M:%S'))
            ]

        # write data
        f.seek(range_from)

        # Add 1 because the range is inclusive

        bytes_to_copy = 1 + range_to - range_from

        buf_length = 64 * 1024

        bytes_copied = 0
        data = ''

        while bytes_copied < bytes_to_copy:

            read_buf = f.read(min(buf_length, bytes_to_copy - bytes_copied))
            print(type(read_buf))

            if len(read_buf) == 0:
                break

            data += read_buf
            # out_file.write(read_buf)

            bytes_copied += len(read_buf)

        response.content = data

        f.close()

        return response


def handle_media_requests(request, response, file_path):
    ctype = guess_type(file_path)
    mime_type = ctype[0]

    response.content_type = mime_type

    try:
        response.status = 200

        with open(file_path, 'rb') as f:
            response.content = f.read()


    except Exception as e:
        response.status = status_not_found
        response.content = ''
        print(e)
        pass

    return response


def handle_image_requests(request, response, file_path):
    ctype = guess_type(file_path)
    mime_type = ctype[0]

    response.content_type = mime_type

    try:
        response.status = 200

        with open(file_path, 'rb') as f:
            response.content = f.read()

    except Exception as e:
        response.status = status_not_found
        response.content = ''
        print(e)
        pass

    return response
